-- usedRustImplement 
-- For soilwork implements, simulates the bare-metal used parts of the implement to rust over time and get bare during work again

-- by modelleicher (https://www.youtube.com/modelleicher) 
-- Version 1.0 -- 19.10.2020 initial implementation for Eberhardt Gazelle 750/11 



usedRustImplement = {};

function usedRustImplement.prerequisitesPresent(specializations)
    return true;
end;

function usedRustImplement.registerEventListeners(vehicleType)
	SpecializationUtil.registerEventListener(vehicleType, "onLoad", usedRustImplement);
	SpecializationUtil.registerEventListener(vehicleType, "onUpdate", usedRustImplement);
	SpecializationUtil.registerEventListener(vehicleType, "onPostLoad", usedRustImplement);
	SpecializationUtil.registerEventListener(vehicleType, "saveToXMLFile", usedRustImplement);
end;


function usedRustImplement:onLoad(savegame)
	self.hourChanged = usedRustImplement.hourChanged;
	self.setUsedRustValue = usedRustImplement.setUsedRustValue;

	g_currentMission.environment:addHourChangeListener(self);

	self.spec_usedRustImplement = {};
	local spec = self.spec_usedRustImplement;
	
	if getXMLBool(self.xmlFile, "vehicle.usedRustImplement.nodes#useIndividualNodes") then
		spec.rustOnlyNodes = {};
		local i = 0;
		while true do
			local node = I3DUtil.indexToObject(self.components, getXMLString(self.xmlFile, "vehicle.usedRustImplement.nodes.rustOnly.node("..i..")#index"), self.i3dMappings);
			if node ~= nil then 
				table.insert(spec.rustOnlyNodes, node);
			else
				break;
			end;
			i = i+1;
		end;
		spec.usedRustNodes = {};
		i = 0;
		while true do
			local node = I3DUtil.indexToObject(self.components, getXMLString(self.xmlFile, "vehicle.usedRustImplement.nodes.wear.node("..i..")#index"), self.i3dMappings);
			if node ~= nil then 
				table.insert(spec.usedRustNodes, node);
			else
				break;
			end;
			i = i+1;		
		end;
	else
	
		spec.usedRustNodes = {};

		for _, component in pairs(self.components) do	
			if component.node ~= nil then
				I3DUtil.getNodesByShaderParam(component.node, "usedColor", spec.usedRustNodes, true)
			end
		end;
		
	end;
	
	-- rustOnly nodes, always set rust color no interpolation
	-- wearNodes, do the usual stuff 

	spec.rustValue = 1;
	
	spec.rustUsedCurve = AnimCurve:new(linearInterpolatorN)
	
	-- time 0 = bare metal, 1 = rust 
	spec.rustUsedCurve:addKeyframe({0.98, 0.98, 0.98 , time = 0 })
	spec.rustUsedCurve:addKeyframe({0.064803, 0.012983, 0.004777 , time = 1 })
	
	spec.rustUsedGlossCurve = AnimCurve:new(linearInterpolatorN)
	
	-- same for gloss 
	spec.rustUsedGlossCurve:addKeyframe({0.9, 0.6, 1.0 , time = 0 })
	spec.rustUsedGlossCurve:addKeyframe({0.5, 0.6, 0.8 , time = 0.7 })
	spec.rustUsedGlossCurve:addKeyframe({0.1, 0.6, 0.2 , time = 1 })	
	
end;

function usedRustImplement:setUsedRustValue(value)
	
	local spec = self.spec_usedRustImplement;
	local valueColor = spec.rustUsedCurve:get(value);
	local valueGloss = spec.rustUsedGlossCurve:get(value);
	for _, node in pairs(spec.usedRustNodes) do
		setShaderParameter(node, "usedColor", valueColor[1], valueColor[2], valueColor[3] , 0, false);
		setShaderParameter(node, "usedGloss", valueGloss[1], valueGloss[2], valueGloss[3] , 0, false);			
	end		
	spec.rustValue = value;

end;

function usedRustImplement:onPostLoad(savegame)
	local spec = self.spec_usedRustImplement

	if spec.usedRustNodes ~= nil then
		local rustValue = spec.rustValue;
		
		if savegame ~= nil then
			rustValue = Utils.getNoNil(getXMLFloat(savegame.xmlFile, savegame.key..".usedRustImplement.rustValue#amount"), rustValue)
		end;
		
		self:setUsedRustValue(rustValue);
	end;
	if spec.rustOnlyNodes ~= nil then
		for _, node in pairs(spec.rustOnlyNodes) do
			setShaderParameter(node, "usedColor", 0.064803, 0.012983, 0.004777 , 0, false);
			setShaderParameter(node, "usedGloss", 0.1, 0.6, 0.2 , 0, false);			
		end		
	end;
end

function usedRustImplement:saveToXMLFile(xmlFile, key)
	local spec = self.spec_usedRustImplement

	if spec.usedRustNodes ~= nil then
		setXMLFloat(xmlFile, key..".rustValue#amount", spec.rustValue)
	end
end


function usedRustImplement:hourChanged()
	local spec = self.spec_usedRustImplement;
	
	local change = 0.005;
	if g_currentMission.environment.weather:getTimeSinceLastRain() < 60 then -- it rained this hour, rust time times 3
		change = change * 3;
	end;
	
	local rustValue = math.min(spec.rustValue + change, 1);
	
	if rustValue ~= spec.rustValue then
		self:setUsedRustValue(rustValue);
	end;
end


function usedRustImplement:onUpdate(dt) 

	if self.isClient then
		local spec = self.spec_usedRustImplement;
	
		local lastSpeed = self:getLastSpeed(true)
		local enabled = self:getDoGroundManipulation() and self:getIsOnField()
		
		if enabled then
			if lastSpeed > 3 then
				local rustValue = math.max(spec.rustValue - 0.0005, 0);
				
				if rustValue ~= spec.rustValue then
					self:setUsedRustValue(rustValue);
				end;			
			end;
		end;
	
	end;
end;


